import glob
import os
import subprocess
from pathlib import Path

from cleo import Command as CLICommand
from ..drivers.MasoniteDriver import MasoniteDriver
from ..drivers.DjangoDriver import DjangoDriver
from ..services.Configuration import Configuration

class StartCommand(CLICommand):
    """
    Starts all the dev sites in the registered directories

    start
        {directory? : The directory you want to start}
    """

    drivers = {
        'masonite': MasoniteDriver,
        'django': DjangoDriver
    }

    def handle(self):
        # Configuration().config()
        if self.argument('directory') and self.argument('directory') == '.':
            site = os.getcwd().split('/')[-1]
            self.start_directory(os.getcwd(), site)
            return
        else:
            self.info('Ensuring All Applications Are Started ..')
            configuration = Configuration()
            for directory in self.get_registered_directories():
                site = directory.split('/')[-2]
                self.start_directory(directory, site)

    def get_home_path(self):
        return str(Path.home())

    def start_directory(self, directory, site):
        configuration = Configuration()
        self.info(f"Starting {site}..")
        venv_config = configuration.get('venvs')
        tld = configuration.get('tld')
        socket_directory = configuration.get('socket_directory')
        if site in venv_config:
            activation_environment = os.path.join(venv_config[site], 'bin/activate')
        else:
            activation_environment = self.find_virtual_environment_activation_file(directory, site)

        driver = self.make(directory)
        command = f"cd {directory}"
        command += f" && source {activation_environment}"
        socket_path = os.path.join(socket_directory, site)
        command += f" && pip install uwsgi && set -m; nohup uwsgi --socket {socket_path}.{tld}.sock --wsgi-file {driver.wsgi_path(directory)} --py-autoreload=1 &> /dev/null &"
        subprocess.run(command, shell=True, close_fds=True,
                        env={'PYTHONPATH': f'{directory}'})

    def get_registered_directories(self):
        directories = []
        for directory in Configuration().get('directories', []):
            directories += glob.glob(os.path.join(directory, '*/'))
        return directories

    def find_virtual_environment_activation_file(self, directory, site):
        return os.path.join(directory, 'venv/bin/activate')

    def make(self, directory):
        for key, available_driver in self.drivers.items():
            selected_driver = available_driver()
            if selected_driver.detect(directory):
                self.info(f'Using driver: {key.capitalize()}')
                return selected_driver
        
        raise ValueError("Could not detect a driver for this project")
        # return self.drivers[driver]()



import os

class MasoniteDriver:

    def wsgi_path(self, directory):
        
        return "wsgi.py"   
    
    def detect(self, directory):
        return os.path.exists(
            os.path.join(directory, 'bootstrap')
        )



import glob
import os
import subprocess
from pathlib import Path

from cleo import Command as CLICommand
from ..drivers.MasoniteDriver import MasoniteDriver
from ..services.Configuration import Configuration

class VenvUnregisterCommand(CLICommand):
    """
    Unregisters the path to the virtualenv directory

    venv:unregister
        {directory? : The directory you want to register the venv directory}
    """

    def handle(self):
        if self.argument('directory'):
            directory = self.argument('directory')
        else:
            directory = os.getcwd()

        configuration = Configuration()

        site = os.getcwd().split('/')[-1]
        
        if 'VIRTUAL_ENV' in os.environ:
            virtual = os.environ['VIRTUAL_ENV']
            self.info(f'Unregistering {virtual} for {site}')
            configuration.remove('venvs', site)
            self.info("Registered")
        else:
            self.info(f'Could not detect virtualenv path')


import glob
import os
import subprocess
from pathlib import Path

from cleo import Command as CLICommand
from ..drivers.MasoniteDriver import MasoniteDriver
from ..services.Configuration import Configuration

class RegisterCommand(CLICommand):
    """
    Registers the current directory to serve Python applications

    register
        {directory? : The directory you want to register}
    """

    def handle(self):
        if self.argument('directory'):
            directory = self.argument('directory')
        else:
            directory = os.getcwd()

        configuration = Configuration()

        self.info(f'Adding current directory: {directory}')
        configuration.set('directories', [directory])
        self.info('Directory Added Successfully')

        self.call("start")


#!/usr/bin/env python
# import subprocess, os, glob
# from pathlib import Path
# home = str(Path.home())

import glob
import os
import subprocess
from pathlib import Path

from cleo import Application
from .commands.StartCommand import StartCommand
from .commands.RegisterCommand import RegisterCommand
from .commands.UnregisterCommand import UnregisterCommand
from .commands.VenvCommand import VenvCommand
from .commands.VenvUnregisterCommand import VenvUnregisterCommand
from .commands.InstallCommand import InstallCommand
from .commands.KillCommand import KillCommand
from .commands.TldCommand import TldCommand
from .commands.ConfigCommand import ConfigCommand


application = Application(name="Masonite Foreman")
application.add(StartCommand())
application.add(RegisterCommand())
application.add(UnregisterCommand())
application.add(VenvCommand())
application.add(InstallCommand())
application.add(KillCommand())
application.add(TldCommand())
application.add(ConfigCommand())

if __name__ == '__main__':
    application.run()



import yaml
from pathlib import Path
import os
class Configuration:

    def get(self, key, default=''):
        return self.config().get(key, default)

    def set(self, key, value):
        config = self.config()
        if key in config and isinstance(value, list):
            if value not in config[key]:
                config[key] += value
                config[key] = list(set(config[key]))
        elif isinstance(value, dict):
            if key in config:
                config[key].update(value)
            else:
                config.update({key: value})
        else:
            config.update({key: value})

        self.save(config)
    
    def remove(self, key, value):
        config = self.config()
        if key in config and isinstance(value, list):
            
            for list_value in value:
                if list_value in value:
                    config[key].remove(list_value)
            config[key] = list(set(config[key]))
        elif isinstance(value, str):
            """
                {'venvs', {
                    'app1': '/some/path'
                }}
            """
            current_dict = self.get(key)
            del current_dict[value]
            config[key] = current_dict
        else:
            pass

        self.save(config)
        
    def save(self, dictionary):
        with open(os.path.join(self.get_home_path(), '.foreman/config.yml'), 'w') as f:
            yaml.dump(dictionary, f)
    
    def config(self):
        with open(os.path.join(self.get_home_path(), '.foreman/config.yml'), 'r') as stream:
            try:
                return yaml.safe_load(stream) or {}
            except yaml.YAMLError as exc:
                print(exc)

    def get_home_path(self):
        return str(Path.home())
